package com.cordova.plugins.cookiemaster;

import android.util.Log;
import android.webkit.CookieManager;

import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.PluginResult;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.net.HttpCookie;


public class CookieMaster extends CordovaPlugin {

    public static final String ACTION_GET_COOKIE_VALUE = "getCookieValue";
    public static final String ACTION_SET_COOKIE_VALUE = "setCookieValue";
    public static final String ACTION_CLEAR_COOKIES = "clearCookies";
    public static final String ACTION_CLEAR_COOKIE = "clearCookie";
    private final String TAG = "CookieMasterPlugin";

    private String serializeCookie(String name, String value, JSONObject options) throws JSONException {
        HttpCookie cookie = new HttpCookie(name, value);
        if (options.has("maxAge")) {
            cookie.setMaxAge(options.getLong("maxAge"));
        }
        if (options.has("domain")) {
            cookie.setDomain(options.getString("domain"));
        }
        if (options.has("path")) {
            cookie.setPath(options.getString("path"));
        }
        if (options.has("secure")) {
            cookie.setSecure(options.getBoolean("secure"));
        }

        String cookieString = cookie.getName() + "=" + cookie.getValue();
        if (cookie.getMaxAge() != -1) {
            cookieString += "; Max-Age=" + cookie.getMaxAge();
        }
        if (cookie.getDomain() != null) {
            cookieString += "; Domain=" + cookie.getDomain();
        }
        if (cookie.getPath() != null) {
            cookieString += "; Path=" + cookie.getPath();
        }
        if (options.optBoolean("httpOnly", false)) {
            cookieString += "; HttpOnly";
        }
        if (cookie.getSecure()) {
            cookieString += "; Secure";
        }
        if (options.has("sameSite")) {
            cookieString += "; SameSite=" + options.getString("sameSite");
        }

        Log.d(TAG, "Internal: " + cookie);
        Log.d(TAG, "Cookie: " + cookieString);
        return cookieString;
    }

    private void setCookieValue(String url, String cookieName, String cookieValue, JSONObject cookieOptions) throws JSONException {
        CookieManager cookieManager = CookieManager.getInstance();
        cookieManager.setCookie(url, serializeCookie(cookieName, cookieValue, cookieOptions));
        cookieManager.flush(); // Sync the cookie to persistent storage.
    }

    @Override
    public boolean execute(String action, JSONArray args, final CallbackContext callbackContext) throws JSONException {

        if (ACTION_GET_COOKIE_VALUE.equals(action)) {
            final String url = args.getString(0);
            final String cookieName = args.getString(1);

            cordova.getThreadPool().execute(new Runnable() {
                public void run() {
                    try {
                        CookieManager cookieManager = CookieManager.getInstance();
                        String[] cookies = cookieManager.getCookie(url).split("; ");
                        String cookieValue = "";

                        for (int i = 0; i < cookies.length; i++) {
                            if (cookies[i].contains(cookieName + "=")) {
                                cookieValue = cookies[i].substring(cookieName.length() + 1).trim();
                                break;
                            }
                        }

                        JSONObject json = null;
                        if (cookieValue != "") {
                            json = new JSONObject("{cookieValue:\"" + cookieValue + "\"}");
                        }
                        if (json != null) {
                            PluginResult res = new PluginResult(PluginResult.Status.OK, json);
                            callbackContext.sendPluginResult(res);
                        } else {
                            callbackContext.error("Cookie not found!");
                        }
                    } catch (Exception e) {
                        Log.e(TAG, "Exception: " + e.getMessage());
                        callbackContext.error(e.getMessage());
                    }
                }
            });
            return true;

        } else if (ACTION_SET_COOKIE_VALUE.equals(action)) {
            final String url = args.getString(0);
            final String cookieName = args.getString(1);
            final String cookieValue = args.getString(2);
            final JSONObject cookieOptions = args.getJSONObject(3);

            cordova.getThreadPool().execute(new Runnable() {
                public void run() {
                    try {
                        setCookieValue(url, cookieName, cookieValue, cookieOptions);
                        PluginResult res = new PluginResult(PluginResult.Status.OK, "Successfully added cookie");
                        callbackContext.sendPluginResult(res);
                    } catch (Exception e) {
                        Log.e(TAG, "Exception: " + e.getMessage());
                        callbackContext.error(e.getMessage());
                    }
                }
            });
            return true;
        } else if (ACTION_CLEAR_COOKIES.equals(action)) {
            CookieManager.getInstance().removeAllCookie();
            callbackContext.success();
            return true;
        } else if (ACTION_CLEAR_COOKIE.equals(action)) {
            final String url = args.getString(0);
            final String cookieName = args.getString(1);

            cordova.getThreadPool().execute(new Runnable() {
                public void run() {
                    try {
                        HttpCookie cookie = new HttpCookie(cookieName, "InvalidCookie");

                        String cookieString = cookie.toString().replace("\"", "");
                        CookieManager cookieManager = CookieManager.getInstance();
                        cookieManager.setCookie(url, cookieString);
                        cookieManager.flush(); // Sync the invalid cookie to persistent storage in order to overwrite the valid cookie.

                        PluginResult res = new PluginResult(PluginResult.Status.OK, "Successfully cleared cookie");
                        callbackContext.sendPluginResult(res);
                    } catch (Exception e) {
                        Log.e(TAG, "Exception: " + e.getMessage());
                        callbackContext.error(e.getMessage());
                    }
                }
            });
            return true;
        }

        callbackContext.error("Invalid action");
        return false;

    }
}
